
   ####################################################################################
   ### 	R program for fitting Bayesian semiparametric density deconvolution models  ### 
   ###			(last modified on June 4, 2013, College Station, TX)				###
   ####################################################################################   
   
   
   	#################
   	### Author(s) ###
   	#################
   
# Author of the codes: Abhra Sarkar
# Authors of the paper: Abhra Sarkar, Bani K. Mallick, John Staudenmayer, Debdeep Pati, Raymond J. Carroll
# Author for correspondence: Raymond J. Carroll (carroll@stat.tamu.edu)

 
 	###########################################
 	### Source Files and Libraries Required ###
 	###########################################
 	
 # The file 'bayesdeconvfuncts.R' must be run and must be in the same working directory.
 # The R libraries 'msm' and 'mvtnorm' must be available.
  
 
	#########################
	### Loading Real Data ### 
	#########################

# Go to the section "Load Real Data". Read the data from a .csv file.
#
# ws 		= 	A vector containing the replicated proxies. 
#				The first m1 elements of the ws vector represents the m1 replicates available for the 1st subject, 
#				the second m2 elements represents the m2 replicates available for the 2nd subject and so on. 
#
# ids 		= 	A vector representing the subject ids. This is also read from the .csv file. 
#				If the .csv file does not have the subject specific ids, the variables 'mis' (a vector containing the number of replicates per subject) 
#				and 'n' (the total no of subjects) have to be separately defined in the section "Load Real Data".
#
# variable.name =	A string for the name of the variable of interest. Required for final storage of results.
# 
# Comments: The methodology does not require the data points to be standardized. 
#			However, to ensure numerical stability, it is recommended that the data be scaled  
#			so that the range of the proxies are within approx 10 to 25 units.
#			For example, the transformation Y = 20*(W-min(W))/(max(W)-min(W)) may be applied.


 
	##############################
	### The Model to Be Fitted ### 
	##############################

# Go to the section "Set Error Structure".
#
# error.fit = 	3 different options are available for 3 different types of models.
#				The option 'normal' fits normally distributed scaled errors,
#				The option 'skewnormal' fits skew-normally distributed scaled errors.
#				The option 'mixture' fits a nonparametric mixture of two component mixture of normals adjusted to have mean zero.


	###############################
	### Setting MCMC Parameters ###
	###############################

# Go to the section "Set MCMC Parameters"
#
# simsize 	=	 Number of MCMC iterations
# burnin 	=	 Number of MCMC iterations rejected as burn-in
#
# Comments: The method may converge with as small as 500 MCMC iterations. For stable results 5000 MCMC iterations with 3000 burn-in is recommended.		
	
	

	###################################
	### Output Generated and Stored ###
	###################################

# x.grid 			= 	The grid on which the denisty is estimated.
# density.x.est 	= 	The estimated density of interest calculated on the grid x.grid 

# e.grid			=   The grid on which the denisty of errors is estimated. Default is seq(-4,4,len=500). To change this go to the section "Priors and Initial Values".
# density.e.est 	= 	The estimated density of errors calculated on the grid e.grid

# var.grid			= 	The grid on which the variance function is estimated
# var.est			= 	The estimated variance function calculated on the grid var.grid

# wbars				= 	The vector of subject-specific sample means
# s2is				= 	The vector of subject-specific sample variances
# density.x0.est	= 	A matrix with columns containing the estimated densities on the grid x.grid for MCMC iterations after the burn-in.


######################################################################################################################################################









rm(list=ls())

source("bayesdeconvfuncts.R")
set.seed(1)



	######################
	### Load Real Data ###
	######################


Data <- read.csv("Data_similar_to_FOLATE_from_EATS.csv",header=TRUE)

ws <- Data$ws
variable.name <- c("FOLATE")

ids <- Data$inds
mis <- as.data.frame(table(ids))$Freq
n <- length(unique(Data$inds))
inds <- rep(1:n,times=mis)








	###########################
	### Set Error Structure ###
	###########################

error.fit <- c("mixture")






	###########################
	### Set MCMC Parameters ###
	###########################

simsize <- 5000
burnin <- 3000






	######################################
	### Add Libraries and Source Files ###
	######################################


# These libraries must be available to R
library(mvtnorm) 
library(msm)







	#################################
	### Priors and Initial Values ###
	#################################

### Initialization and prior of xs and us

wbars <- as.vector(tapply(ws,inds,"mean")) 
s2is <- as.vector(tapply(ws,inds,var))


xs <- current.xs <- start.xs <- as.vector(wbars)
us <- ws - rep(xs,times=mis)
x.grid <- seq(min(wbars),max(wbars),length=500)



alpha.x=0.1	# robust but should not be set too large or small, values in [0.0001,1] produces good results

# Normal-Inverse-Scaled-Chisquare
nu0.x = 1/5
gama0.x = 3	                  			# Note the spelling, also note that this must be greater than 2
mu0.x = mean(xs)
sigmasq0.x = var(xs)/((1+1/nu0.x)*(gama0.x/(gama0.x-2)))
student.x.grid = (x.grid-mu0.x)/(sqrt(sigmasq0.x)*sqrt(1+1/nu0.x))

z.x <- 1:n								# one separate clusters for each obs
mu.x <- xs								# unique values
sigmasq.x <- rep(var(xs)/5,n)			# unique values



### Prior and initialization of s2t and thetas

alpha.t <- .1
beta.t <- .1

s2t <- current.s2t <- start.s2t <- 0.1

K.t <- 10 				# num of knots, results are robust to variations in values
P.t <- P.mat(K.t+1) 	# penalty matrix
range.start.xs <- diff(range(start.xs))
knots.t <- seq(min(start.xs)-.1*range.start.xs,
             max(start.xs)+.1*range.start.xs,
             length=(K.t))
optim_results <- optim(rep(1,K.t+1), fr, gr, method = "BFGS")
thetas <- current.thetas <- start.thetas <- optim_results$par
sig.tune.thetas.1 <- 0		# results are very robust to these two tuning parameters
sig.tune.thetas.2 <- 0.1	
prop.sig.thetas <- prop.sig.thetas.fn(start.thetas,start.s2t)

var.grid <- seq(min(wbars),max(wbars),length=100)
vars <- current.vars <- B.basis(xs,knots.t)%*%exp(current.thetas)
vfix <- exp(thetas[2])+exp(thetas[3])


### Prior and initialization of skewness
e.grid <- seq(-4,4,length=100)
if(error.fit=="skewnormal")
	{
	mu0.skewness <- 0
	sigmasq0.skewness <- 16
	skewness <- current.skewness <- start.skewness <- 0 
	}

### Prior and initialization for mixture
if(error.fit=="mixture")
	{	
	a.u <- 1
	b.u <- 1
	sigma0.u <- 3
	simsize.mh.u <- 20
	z.u <- rep(1,sum(mis))
	k.u <- 1
	alpha.u <- 0.1	# robust but should not be set too large or small
	params.u <- matrix(c(0.5,0,1,1),nrow=20,ncol=4,byrow=T)		# unique values
	density.e.est <- numeric(length(e.grid))
	accepted <- numeric(simsize)
	}


	





	###############################
	### Storage for MCMC Output ###
	###############################


density.x.est <- numeric(length(x.grid))
density.x0.est <- matrix(0,length(x.grid),simsize-burnin)
var.est <- numeric(length(var.grid))
e.grid <- seq(-4,4,length=100)
density.e.est <- numeric(length(e.grid))
proposed.log.prior <- current.log.prior <- 0
proposed.log.proposal <- current.log.proposal <- 0
no.clusters <- numeric(simsize)













	##################
	### Start MCMC ###
	##################



for (iii in 1:simsize)
	{
	if(iii%%100==0)
		print(iii)



	### Updating z.x
	
	student.x = (xs-mu0.x)/(sqrt(sigmasq0.x)*sqrt(1+1/nu0.x))
	marginal.tpdfs = dt(student.x,df=gama0.x)
	
	for(ii in 1:n)
		{
		prob.x = tabulate(z.x) 
        	k.x = length(unique(z.x[-ii]))
        	if((prob.x[z.x[ii]]==1)&&(z.x[ii]<max(z.x)))    # z.x[ii] appears only once in z.x
			{
			temp.mu.x = mu.x[z.x[ii]]
			temp.sigmasq.x = sigmasq.x[z.x[ii]]
			for(jj in z.x[ii]:k.x)
				{
				mu.x[jj] = mu.x[jj+1]
				sigmasq.x[jj] = sigmasq.x[jj+1]
				}
			z.x[z.x > z.x[ii]] = z.x[z.x > z.x[ii]]-1
			z.x[ii] = k.x+1         # Necessary steps
			mu.x[k.x+1] = temp.mu.x
			sigmasq.x[k.x+1] = temp.sigmasq.x  
			}
        	prob.minus.x = tabulate(z.x[-ii])
		for(kk in 1:k.x)
			{
			likelihood = dnorm(xs[ii],mu.x[kk],sqrt(sigmasq.x[kk]))
			prob.x[kk] = prob.minus.x[kk] * likelihood
			}
		prob.x[k.x+1] = alpha.x * marginal.tpdfs[ii]
		prob.x = prob.x/sum(prob.x)    		# NOT really necessary        
		z.x[ii] = sample(k.x+1,1,TRUE,prob.x)   	# New z.x[ii] drawn
		if(z.x[ii]==(k.x+1))
			{
			#sigmasq.x[k.x+1] = 1/rgamma(1,shape=gama0.x,rate=sigmasq0.x)			# Normal-Inverse-Gamma
			sigmasq.x[k.x+1] = 1/rgamma(1,shape=gama0.x/2,rate=gama0.x*sigmasq0.x/2)	# Normal-Inverse-Scaled-Chisquare
			mu.x[k.x+1] = rnorm(1,mu0.x,sqrt(sigmasq.x[k.x+1]/nu0.x))
			}
		}
    
    
	### Updating mu.x, sigmsq.x
	
	k.x = no.clusters[iii] = max(z.x)                # Number of clusters
	for(kk in 1:k.x)
		{
		temp = which(z.x==kk)
		xspool = xs[temp]

		nutemp.x = nu0.x + length(xspool)
		mutemp.x = (nu0.x*mu0.x + sum(xspool)) / (nutemp.x)
		gamatemp.x = gama0.x + length(xspool)				# Normal-Inverse-Scaled-Chisquare
		sigmasqtemp.x = (gama0.x*sigmasq0.x + sum(xspool^2) + nu0.x*mu0.x^2 - nutemp.x*mutemp.x^2) / gamatemp.x	
		sigmasq.x[kk] = 1/rgamma(1,shape=gamatemp.x/2,rate=(gamatemp.x*sigmasqtemp.x)/2)
		mu.x[kk] = rnorm(1,mutemp.x,sqrt(sigmasq.x[kk]/nutemp.x))
		}




	if(error.fit=="normal")
		{
		### Updating xs (and us)

		proposed.xs = rtnorm(n,mean=current.xs,sd=diff(range(start.xs))/6,lower=min(knots.t),upper=max(knots.t))
		proposed.vars = B.basis(proposed.xs,knots.t)%*%exp(thetas)

		prob.x = (tabulate(z.x)+alpha.x)/(alpha.x+n)
		proposed.prior = current.prior = numeric(n)
		for(kk in 1:k.x)
			{
			proposed.prior = proposed.prior + prob.x[kk]*dnorm(proposed.xs,mu.x[kk],sqrt(sigmasq.x[kk]))
			current.prior = current.prior + prob.x[kk]*dnorm(current.xs,mu.x[kk],sqrt(sigmasq.x[kk]))
			}

		mh.ratio = proposed.prior/current.prior
		mh.ratio = (   mh.ratio * dtnorm(current.xs,mean=proposed.xs,sd=diff(range(start.xs))/6,lower=min(knots.t),upper=max(knots.t))
					/ dtnorm(proposed.xs,mean=current.xs,sd=diff(range(start.xs))/6,lower=min(knots.t),upper=max(knots.t))   )
		
		temp.current.likelihood = dnorm(ws,mean=rep(current.xs,times=mis),sd=rep(sqrt(current.vars),times=mis))
		temp.proposed.likelihood = dnorm(ws,mean=rep(proposed.xs,times=mis),sd=rep(sqrt(proposed.vars),times=mis))
		
		current.likelihood = tapply(temp.current.likelihood,inds,"prod")
		proposed.likelihood = tapply(temp.proposed.likelihood,inds,"prod")

		mh.ratio = mh.ratio * proposed.likelihood/current.likelihood	
		mh.ratio[is.nan(mh.ratio)] = 0

		u = runif(n)
		inds.to.replace =(1:n)[u<mh.ratio]
		xs[inds.to.replace] = current.xs[inds.to.replace] = proposed.xs[inds.to.replace]
		vars[inds.to.replace] = current.vars[inds.to.replace] = proposed.vars[inds.to.replace]

		us = ws - rep(xs,times=mis)


		### Updating theta

		proposed.thetas = rmvnorm(1,current.thetas,diag(rep(sig.tune.thetas.1,(K.t+1)))+sig.tune.thetas.2*prop.sig.thetas)
		proposed.thetas = t(proposed.thetas)
		proposed.vars = B.basis(xs,knots.t)%*%exp(proposed.thetas)

		current.log.prior = - t(current.thetas)%*%P.t%*%current.thetas/(2*s2t)
		proposed.log.prior = - t(proposed.thetas)%*%P.t%*%proposed.thetas/(2*s2t)

		temp.current.likelihood = dnorm(us,mean=rep(0,times=sum(mis)),sd=rep(sqrt(current.vars),times=mis))
		temp.proposed.likelihood = dnorm(us,mean=rep(0,times=sum(mis)),sd=rep(sqrt(proposed.vars),times=mis))
		
		current.log.likelihood = sum(log(temp.current.likelihood))
		proposed.log.likelihood = sum(log(temp.proposed.likelihood))

		log.mh.ratio = proposed.log.prior + proposed.log.likelihood - current.log.likelihood - current.log.prior

		log.u = log(runif(1))
		if(log.u<log.mh.ratio)
			{
			thetas = current.thetas = proposed.thetas
			vars = current.vars = proposed.vars
			}
		}





	if(error.fit=="skewnormal")
		{
		### Updating xs (and us)

		proposed.xs = rtnorm(n,mean=current.xs,sd=diff(range(start.xs))/6,lower=min(knots.t),upper=max(knots.t))
		proposed.vars = B.basis(proposed.xs,knots.t)%*%exp(thetas)

		prob.x = tabulate(z.x)/(n)
		proposed.prior = current.prior = numeric(n)
		for(kk in 1:k.x)
			{
			proposed.prior = proposed.prior + prob.x[kk]*dnorm(proposed.xs,mu.x[kk],sqrt(sigmasq.x[kk]))
			current.prior = current.prior + prob.x[kk]*dnorm(current.xs,mu.x[kk],sqrt(sigmasq.x[kk]))
			}

		mh.ratio = proposed.prior/current.prior
		mh.ratio = (   mh.ratio * dtnorm(current.xs,mean=proposed.xs,sd=diff(range(start.xs))/6,lower=min(knots.t),upper=max(knots.t))
					/ dtnorm(proposed.xs,mean=current.xs,sd=diff(range(start.xs))/6,lower=min(knots.t),upper=max(knots.t))   )

		temp.current.likelihood = dskewnorm(ws,mean=rep(current.xs,times=mis),sd=rep(sqrt(current.vars),times=mis),skewness)
		temp.proposed.likelihood = dskewnorm(ws,mean=rep(proposed.xs,times=mis),sd=rep(sqrt(proposed.vars),times=mis),skewness)
		
		current.likelihood = tapply(temp.current.likelihood,inds,"prod")
		proposed.likelihood = tapply(temp.proposed.likelihood,inds,"prod")

		mh.ratio = mh.ratio * proposed.likelihood/current.likelihood	
		mh.ratio[is.nan(mh.ratio)] = 0

		u = runif(n)
		inds.to.replace =(1:n)[u<mh.ratio]
		xs[inds.to.replace] = current.xs[inds.to.replace] = proposed.xs[inds.to.replace]
		vars[inds.to.replace] = current.vars[inds.to.replace] = proposed.vars[inds.to.replace]

		us = ws - rep(xs,times=mis)


		### Updating theta
	
		proposed.thetas = rmvnorm(1,current.thetas,(diag(rep(sig.tune.thetas.1,(K.t+1)))+sig.tune.thetas.2*prop.sig.thetas))
		proposed.thetas = t(proposed.thetas)
		proposed.vars = B.basis(xs,knots.t)%*%exp(proposed.thetas)

		current.log.prior = - t(current.thetas)%*%P.t%*%current.thetas/(2*s2t)
		proposed.log.prior = - t(proposed.thetas)%*%P.t%*%proposed.thetas/(2*s2t)

		temp.current.likelihood = dskewnorm(us,mean=rep(0,times=sum(mis)),sd=rep(sqrt(current.vars),times=mis),skewness)
		temp.proposed.likelihood = dskewnorm(us,mean=rep(0,times=sum(mis)),sd=rep(sqrt(proposed.vars),times=mis),skewness)

		current.log.likelihood = sum(log(temp.current.likelihood))
		proposed.log.likelihood = sum(log(temp.proposed.likelihood))

		log.mh.ratio = proposed.log.prior + proposed.log.likelihood - current.log.likelihood - current.log.prior

		log.u = log(runif(1))
		if(log.u<log.mh.ratio)
			{
			thetas = current.thetas = proposed.thetas
			vars = current.vars = proposed.vars
			}
	

		### Updating skewness

		proposed.skewness <- rnorm(1,current.skewness,sd=1)

		current.log.prior = - (current.skewness-mu0.skewness)^2/(2*sigmasq0.skewness)
		proposed.log.prior = - (proposed.skewness-mu0.skewness)^2/(2*sigmasq0.skewness)

		temp.current.likelihood = dskewnorm(us,mean=rep(0,times=sum(mis)),sd=rep(sqrt(vars),times=mis),current.skewness)
		temp.proposed.likelihood = dskewnorm(us,mean=rep(0,times=sum(mis)),sd=rep(sqrt(vars),times=mis),proposed.skewness)
		
		current.log.likelihood = sum(log(temp.current.likelihood))
		proposed.log.likelihood = sum(log(temp.proposed.likelihood))

		log.mh.ratio = proposed.log.prior + proposed.log.likelihood - current.log.likelihood - current.log.prior
		
		log.u = log(runif(1))
		if(log.u<log.mh.ratio)
			skewness = current.skewness = proposed.skewness
		}





	if(error.fit=="mixture")
		{
		### Updating xs (and us)

		proposed.xs = rtnorm(n,mean=current.xs,sd=diff(range(start.xs))/6,lower=min(knots.t),upper=max(knots.t))
		proposed.vars = B.basis(proposed.xs,knots.t)%*%exp(thetas)
		
		prob.x = tabulate(z.x)/(n)
		proposed.prior = current.prior = numeric(n)
		for(kk in 1:k.x)
			{
			proposed.prior = proposed.prior + prob.x[kk]*dnorm(proposed.xs,mu.x[kk],sqrt(sigmasq.x[kk]))
			current.prior = current.prior + prob.x[kk]*dnorm(current.xs,mu.x[kk],sqrt(sigmasq.x[kk]))
			}

		mh.ratio = proposed.prior/current.prior
		mh.ratio = (   mh.ratio * dtnorm(current.xs,mean=proposed.xs,sd=diff(range(start.xs))/6,lower=min(knots.t),upper=max(knots.t))
					/ dtnorm(proposed.xs,mean=current.xs,sd=diff(range(start.xs))/6,lower=min(knots.t),upper=max(knots.t))   )

		temp.current.likelihood = d.restricted.mix.norm(ws,mean=rep(current.xs,times=mis),sd=rep(sqrt(current.vars),times=mis),params.u[z.u,])
		temp.proposed.likelihood = d.restricted.mix.norm(ws,mean=rep(proposed.xs,times=mis),sd=rep(sqrt(proposed.vars),times=mis),params.u[z.u,])
		
		current.likelihood = tapply(temp.current.likelihood,inds,"prod")
		proposed.likelihood = tapply(temp.proposed.likelihood,inds,"prod")

		mh.ratio = mh.ratio * proposed.likelihood/current.likelihood	
		mh.ratio[is.nan(mh.ratio)] = 0

		u = runif(n)
		inds.to.replace =(1:n)[u<mh.ratio]
		xs[inds.to.replace] = current.xs[inds.to.replace] = proposed.xs[inds.to.replace]
		vars[inds.to.replace] = current.vars[inds.to.replace] = proposed.vars[inds.to.replace]

		us = ws - rep(xs,times=mis)
		

		### Updating theta

		proposed.thetas = rmvnorm(1,current.thetas,prop.sig.thetas)
		while(proposed.thetas[3]>log(vfix))
			proposed.thetas = rmvnorm(1,current.thetas,sig.tune.thetas.1 * prop.sig.thetas)
		proposed.thetas[2] = log(vfix-exp(proposed.thetas[3]))
 		proposed.thetas = t(proposed.thetas)
		proposed.vars = B.basis(xs,knots.t)%*%exp(proposed.thetas)
		
		current.log.prior = - t(current.thetas)%*%P.t%*%current.thetas/(2*s2t)
		proposed.log.prior = - t(proposed.thetas)%*%P.t%*%proposed.thetas/(2*s2t)

		temp.current.likelihood = d.restricted.mix.norm(us,mean=rep(0,times=sum(mis)),sd=rep(sqrt(current.vars),times=mis),params.u[z.u,])
		temp.proposed.likelihood = d.restricted.mix.norm(us,mean=rep(0,times=sum(mis)),sd=rep(sqrt(proposed.vars),times=mis),params.u[z.u,])

		current.log.likelihood = sum(log(temp.current.likelihood))
		proposed.log.likelihood = sum(log(temp.proposed.likelihood))

		log.mh.ratio = proposed.log.prior + proposed.log.likelihood - current.log.likelihood - current.log.prior

		log.u = log(runif(1))
		if(log.u<log.mh.ratio)
			{
			thetas = current.thetas = proposed.thetas
			vars = current.vars = proposed.vars
			}



		### Updating z.u

		for(ii in 1:sum(mis))
			{
			prob.u = tabulate(z.u) 
	        	k.u = length(unique(z.u[-ii]))
      	  	if((prob.u[z.u[ii]]==1)&&(z.u[ii]<max(z.u)))    # z.u[ii] appears only once in z.u
				{
				temp.params.u = params.u[z.u[ii],]
				for(jj in z.u[ii]:k.u)         	# Necessary step			
					params.u[jj,] = params.u[jj+1,]
				z.u[z.u > z.u[ii]] = z.u[z.u > z.u[ii]]-1
				z.u[ii] = k.u+1         		# Necessary step 
				params.u[k.u+1,] = temp.params.u
				}

			prob.minus.u <- tabulate(z.u[-ii])
			prob.minus.u[k.u+1] = alpha.u
			prob.u = prob.minus.u/sum(prob.minus.u)    # NOT really necessary

			proposed.z.u = sample(k.u+1,1,TRUE,prob.u)   # z.u[ii] proposed

			if(z.u[ii]!=proposed.z.u)
				{
				if(proposed.z.u==(k.u+1))
					proposed.params.u <- r.proposal.params.restricted.mix.norm(1,1,3,3,2.5,3,2.5)
				else
					proposed.params.u <- params.u[proposed.z.u,]
				
				proposed.likelihood = d.restricted.mix.norm(us[ii],mean=0,sd=sqrt(current.vars[inds[ii]]),proposed.params.u)
				current.likelihood = d.restricted.mix.norm(us[ii],mean=0,sd=sqrt(current.vars[inds[ii]]),params.u[z.u[ii],])
				acc.prob <- proposed.likelihood/current.likelihood
				if(runif(1)<acc.prob)
					{
					z.u[ii] <- proposed.z.u
					params.u[z.u[ii],] <- proposed.params.u
					}
				}
			}



		### Updating params.u
	        
		k.u = max(z.u)                # Number of clusters
		if(iii>2000)
			simsize.mh.u = 1
		for(rr in 1:simsize.mh.u)
			{
			for(kk in 1:k.u)
				{
				temp = which(z.u==kk)
				uspool = us[temp]
				varspool = vars[inds[temp]]

				proposed.params.u <- r.tnorm.proposal.params.restricted.mix.norm(params.u[kk,])
				
				proposed.log.likelihood = sum(log(d.restricted.mix.norm(uspool,mean=0,sd=sqrt(varspool),proposed.params.u)))
				current.log.likelihood = sum(log(d.restricted.mix.norm(uspool,mean=0,sd=sqrt(varspool),params.u[kk,])))
			
				proposed.log.prior = log(dnorm(proposed.params.u[2],mean=0,sd=sigma0.u))+sum(log(dgamma(1/proposed.params.u[3:4],shape=a.u,scale=b.u)))
				current.log.prior = log(dnorm(params.u[kk,2],mean=0,sd=sigma0.u))+sum(log(dgamma(1/params.u[kk,3:4],shape=a.u,scale=b.u)))
				
				proposed.log.proposal = log(dtnorm(params.u[kk,1],mean=proposed.params.u[1],sd=0.01))+sum(log(dtnorm(params.u[kk,3:4],mean=proposed.params.u[3:4],sd=0.1,lower=0,upper=Inf)))
				current.log.proposal = log(dnorm(proposed.params.u[1],mean=params.u[kk,1],sd=0.01))+sum(log(dtnorm(proposed.params.u[3:4],mean=params.u[kk,3:4],sd=0.1,lower=0,upper=Inf)))

				log.acc.prob <- (proposed.log.likelihood-current.log.likelihood) + (proposed.log.prior-current.log.prior) + (proposed.log.proposal-current.log.proposal)
				# Only the likelihood term dominates. To improve speed, the second and the third terms can be neglected. 
				
				if(log(runif(1))<log.acc.prob)
					{
					params.u[kk,] <- proposed.params.u
					accepted[iii] <- 1
					}
				} 
			}
		}






	### Updating s2t

	s2t = 1/rgamma(1,shape=alpha.t+(K.t+1)/2,rate=beta.t+t(thetas)%*%P.t%*%thetas/2)

	




	if(iii>burnin)
		{
		prob.x = tabulate(z.x)/(alpha.x+n)
		for(kk in 1:k.x)
			density.x.est = density.x.est + prob.x[kk]*dnorm(x.grid,mu.x[kk],sqrt(sigmasq.x[kk]))
		prob.x[k.x+1] = alpha.x/(alpha.x+n)
		density.x.est = density.x.est + prob.x[k.x+1]*dt(student.x.grid,df=gama0.x)
		for(ii in 1:length(x.grid))
			density.x0.est[ii,iii-burnin] = sum(prob.x[1:k.x]*dnorm(x.grid[ii],mu.x[1:k.x],sqrt(sigmasq.x[1:k.x])))+prob.x[k.x+1]*dt(student.x.grid[ii],df=gama0.x)

		if(error.fit!="mixture")
			var.est = var.est + B.basis(var.grid,knots.t)%*%exp(thetas)

		if(error.fit=="skewnormal")
			{
			density.e.est = density.e.est + dskewnorm(e.grid,mean=rep(0,times=length(e.grid)),rep(1,times=length(e.grid)),skewness)
			}

		if(error.fit=="mixture")
			{
			k.u = max(z.u)                # Number of clusters
			prob.e <- tabulate(z.u)/sum(mis)
			var.e = var.e.fn(prob.e[1:k.u],params.u[1:k.u,])
			var.est = var.est + B.basis(var.grid,knots.t)%*%exp(thetas) * var.e
			density.e.est <- density.e.est + d.scaled.restricted.mix.norm(e.grid,0,1,prob.e[1:k.u],params.u[1:k.u,])
			}
		}
	
	}






density.x.est <- density.x.est/(simsize-burnin)
var.est <- var.est/(simsize-burnin)
if(error.fit=="normal")
	density.e.est <- dnorm(e.grid)
if(error.fit!="normal")
	density.e.est <- density.e.est/(simsize-burnin)





if(TRUE)
	{	
	#filename <- paste("EATS_Data_Simulated_From_Fits", variable.name,".pdf",sep="")
	#pdf(filename,width=8,height=8)
	
	def.par <- par(no.readonly = TRUE)	
	mat=matrix(c(1,1,1,1,2,3),nrow=3,ncol=2,byrow=T)
	layout(mat)
	
	plot(x.grid[1:300],density.x.est[1:300],ylim=c(0,max(density.x.est)),type="l",col="blue")
	#plot(x.grid,density.x.est,ylim=c(0,max(density.x.est)),type="l",col="blue")
	title(main=paste("Eats_Data",variable.name,"Estimated_Density",sep="_"))

	plot(e.grid,density.e.est,type="l",col="blue")
	points(e.grid,dnorm(e.grid),ylim=c(0,max(density.e.est)),type="l",lty=2,col="red")
	title(main=paste("Eats_Data",variable.name,error.fit,sep="_"))

	plot(var.grid[1:50],var.est[1:50],ylim=c(0,max(var.est[1:50])),type="l",col="blue")
	#plot(var.grid,var.est,ylim=c(0,max(var.est)),type="l",col="blue")
	points(wbars,s2is,xlim=c(min(var.grid),max(var.grid)),pch="*")
	title(main=paste("Eats_Data",variable.name,"Variance_Function",sep="_"))

	par(def.par)
	
	#dev.off()
	}



if(FALSE)
	{
	filename <- paste("Eats_Data_Simulated_From_Fits",variable.name,error.fit,simsize,burnin,sep="_")
	filename <- paste(filename,".RData",sep="")
	save(x.grid,density.x.est,e.grid,density.e.est,var.grid,var.est,wbars,s2is,density.x0.est,file=filename)
	}








